#include <Application.h>
#include <Roster.h>
#include <stdio.h>
#include <stdlib.h>
#include <TextControl.h>
#include <Window.h>
#include <InterfaceKit.h>
#include <Button.h>
#include <String.h>
#include <StringView.h>
#include <Box.h>
#include <View.h>
#include "SoundPlay Suenos.h"


void *getplugin(void **data,const char *name, const char *, uint32, plugin_info * );
void destroyplugin(void*,void *_loader);
status_t show(void*);
void hide(void*);
void setconfig(void*,BMessage *config);
void getconfig(void*,BMessage *config);

static plugin_descriptor plugin={
	PLUGIN_DESCRIPTOR_MAGIC,PLUGIN_DESCRIPTOR_VERSION,
	PLUGINID,1, PLUGIN_IS_INTERFACE,

	"SoundPlay Suenos",
	"Version: 0.2\n\nSweet Dreams\n\nCopyleft (L) Tim Rick.\nNo rights reserved.\n",
	NULL,
	NULL,
	&getplugin,
	&destroyplugin
};

static interface_plugin_ops plugin_ops={
	PLUGIN_INTERFACE_MAGIC, PLUGIN_INTERFACE_VERSION,
	show,
	hide,
	getconfig
};

static plugin_descriptor *plugs[]={
	&plugin,0
};

plugin_descriptor **get_plugin_list(void)
{
	return plugs;
}



void *getplugin(void **data,const char *name, const char *, uint32, plugin_info *info )
{
	SoundPlaySuenosWindow *win=new SoundPlaySuenosWindow(info->controller);
	info->controller->AddWindow(win,FILTER_HOTKEYS); // let SoundPlay handle some hotkeys for this window
	*data=win;
	return &plugin_ops;
}

void destroyplugin(void *,void *data)
{
	SoundPlaySuenosWindow *win=(SoundPlaySuenosWindow*)data;
	win->Lock();
	win->Quit();
}

status_t show(void *data)
{
	SoundPlaySuenosWindow *win=(SoundPlaySuenosWindow*)data;
	win->Show();
	return B_OK;
}

void hide(void *data)
{
	SoundPlaySuenosWindow *win=(SoundPlaySuenosWindow*)data;
	win->Hide();
}

void setconfig(void *data,BMessage *config)
{
/*	SoundPlaySuenosWindow *win=(SoundPlaySuenosWindow*)data;
	BRect rect;
	if(B_OK==config->FindRect("frame",&rect))
	{
		win->MoveTo(rect.LeftTop());
		win->ResizeTo(rect.Width(),rect.Height());
	}*/
}
void getconfig(void *data,BMessage *config)
{
/*	SoundPlaySuenosWindow *win=(SoundPlaySuenosWindow*)data;
	config->MakeEmpty();
	config->AddRect("frame",win->Frame());*/
}


SoundPlaySuenosWindow::SoundPlaySuenosWindow(SoundPlayController *controller)
	: BWindow(BRect(100,250,268,375),"SoundPlay Suenos",B_TITLED_WINDOW, B_NOT_ZOOMABLE | B_NOT_RESIZABLE | B_ASYNCHRONOUS_CONTROLS)
{
	fallingAsleep = false;
	ctrl=controller;
	
	BBox *background = new BBox(Bounds(), "", B_FOLLOW_ALL_SIDES, B_WILL_DRAW |
		B_FRAME_EVENTS | B_NAVIGABLE_JUMP, B_PLAIN_BORDER);
	AddChild(background);
	background->SetViewColor(215, 215, 215);
	
	BRect minutesUntilSleepRect;
	minutesUntilSleepRect.left = 27;
	minutesUntilSleepRect.top = 40;
	minutesUntilSleepRect.right = 130;
	minutesUntilSleepRect.bottom = 50;
	minutesUntilSleep = new BTextControl(minutesUntilSleepRect, "T - X and Dreaming", "",
		"", new BMessage('MINS'));
	minutesUntilSleep->SetDivider(0);
	minutesUntilSleep->SetEnabled(true);
	background->AddChild(minutesUntilSleep);
	minutesUntilSleep->SetText("60");
	
	const BRect kButtonFrame (29, 80, 130, 90);
	GoToBedButton = new BButton(kButtonFrame,GOTOSLEEP,GOTOSLEEP,
					new BMessage(BUTTON_MSG));
	background->AddChild(GoToBedButton);

	BRect rectum(30,10,160,30);
	BStringView *sleepLabel = new BStringView(rectum,"Sleep Label","Minutes until sleep");
	background->AddChild(sleepLabel);
}

void SoundPlaySuenosWindow::Show()
{
	ctrl->Lock();

	// Cache the two playlists/tracks
	// It is important to realize that once you have a valid playlist,
	// it will never become invalid. Its controls might go away, but
	// the playlist itself will remain valid.
	track=ctrl->PlaylistAt(0);
	ctrl->Unlock();
	BWindow::Show();
}

SoundPlaySuenosWindow::~SoundPlaySuenosWindow()
{
	ctrl->Lock();
	if(track->CountItems()==0)
		ctrl->RemovePlaylist(track);
	ctrl->Unlock();
}

void SoundPlaySuenosWindow::MessageReceived(BMessage *mes)
{
	// re-add the playlists to the window in case they got orphaned
	ctrl->Lock();
	if(!track->HasControls())
		track->AddControls();
	ctrl->Unlock();

	// For simplicity, we lock the playlists here and unlock at the end
	// of the function, instead of around function that requires them to
	// be locked.
	track->Lock();
	
	switch(mes->what)
	{
		case BUTTON_MSG:
				{
					if(!fallingAsleep) {
						char *hours;
						hours = (char *)minutesUntilSleep->Text();
						StartCountdown(atoi((char *)minutesUntilSleep->Text()));
						GoToBedButton->SetLabel(STAYUPLATE);
					} else {
						StopCountdown();
						GoToBedButton->SetLabel(GOTOSLEEP);
					}
				}
				break;
	/*	case B_SIMPLE_DATA:
			// file dropped
			// to keep things simple, we simply determine in which half
			// of the window it was dropped (*if* it was dropped)
			if(mes->WasDropped())
			{
				entry_ref ref;
				if(B_OK==mes->FindRef("refs",&ref))
				{
					BPoint point=mes->DropPoint();
					point=ConvertFromScreen(point);
					PlaylistPtr track;
					track->Lock();
					track->MakeEmpty();
					track->Add(ref);
					track->PlayFile(0);
					track->Unlock();
				}
			}
			break;*/
		default:
			BWindow::MessageReceived(mes);
			break;
	}
	track->Unlock();
}

bool SoundPlaySuenosWindow::QuitRequested()
{
	ctrl->DisableInterface(PLUGINID);
	return false;
}

void SoundPlaySuenosWindow::StartCountdown(float minutes) {
	fallingAsleep = true;
	currentVolume = track->Volume();
	float nap = 0.0;
	nap = 1000000 * 1.0 * 60.0 * minutes * 0.01;
	napForMicroseconds = (bigtime_t)nap;
	fThreadID = spawn_thread(FallingAsleepThreadFunc, "fallingasleep_thread", B_LOW_PRIORITY, this);
	resume_thread(fThreadID);
}

bool SoundPlaySuenosWindow::StopCountdown() {
	kill_thread(fThreadID);
	fallingAsleep = false;
	return true;
}

void SoundPlaySuenosWindow::LowerVolume(float thismuch) {
	if(fallingAsleep) {
		if(currentVolume > 0.0) {
			currentVolume = (track->Volume() - thismuch);
			track->SetVolume(currentVolume);	
		} else {
			fallingAsleep = false;
		}
	}
}

int32 SoundPlaySuenosWindow::FallingAsleepThreadFunc(void *arg)
{	
	SoundPlaySuenosWindow *win = (SoundPlaySuenosWindow *)arg;

	while(win->fallingAsleep) {
		snooze(win->napForMicroseconds);
		win->LowerVolume(0.01);
	}	
	return B_OK;
}

int main()
{
	BApplication app("application/x-vnd.marcone-spsuenos");
	
	BMessage msg(ENABLE_INTERFACE);
	msg.AddString("interface",PLUGINID);
	be_roster->Launch("application/x-vnd.marcone-soundplay",&msg);
	return 0;
}

